 SUBROUTINE compute_vpt

!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
!
! Former revisions:
! -----------------
! $Id: compute_vpt.f90 1310 2014-03-14 08:01:56Z raasch $
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 803 2012-01-16 15:48:46Z franke
! Bugfix: wrong factor in calculation of vpt in case of cloud droplets
!
! 799 2011-12-21 17:48:03Z franke
! Bugfix: ql is now included in calculation of vpt in case of 
!         cloud droplets 
!
! RCS Log replace by Id keyword, revision history cleaned up
!
! Revision 1.5  2001/03/30 06:58:52  raasch
! Translation of remaining German identifiers (variables, subroutines, etc.)
!
! Revision 1.1  2000/04/13 14:40:53  schroeter
! Initial revision
!
!
! Description:
! -------------
! Computation of the virtual potential temperature 
!-------------------------------------------------------------------------------!

    USE arrays_3d
    USE indices
    USE cloud_parameters
    USE control_parameters

    IMPLICIT NONE

    INTEGER :: k

    IF ( .NOT. cloud_physics .AND. .NOT. cloud_droplets ) THEN
       vpt = pt * ( 1.0 + 0.61 * q )
    ELSE IF (cloud_physics) THEN
       DO  k = nzb, nzt+1
          vpt(k,:,:) = ( pt(k,:,:) + pt_d_t(k) * l_d_cp * ql(k,:,:) ) * &
                       ( 1.0 + 0.61 * q(k,:,:) - 1.61 * ql(k,:,:) ) 
       ENDDO
    ELSE
       vpt = pt * ( 1.0 + 0.61 * q - ql ) 
    ENDIF

 END SUBROUTINE compute_vpt
