 SUBROUTINE local_tremain_ini

!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
!
! Former revisions:
! -----------------
! $Id: local_tremain_ini.f90 1310 2014-03-14 08:01:56Z raasch $
!
! 1092 2013-02-02 11:24:22Z raasch
! unused variables removed
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 225 2009-01-26 14:44:20Z raasch
! Type of count and count_rate changed to INTEGER(8) in order to avoid out of
! range problems (which result in measured negative time intervals)
!
! 82 2007-04-16 15:40:52Z raasch
! Cpp-directive lctit renamed lc
!
! RCS Log replace by Id keyword, revision history cleaned up
!
! Revision 1.13  2007/02/11 13:07:03  raasch
! Allowed cpu limit is now read from file instead of reading the value from
! environment variable (see routine parin)
!
! Revision 1.1  1998/03/18 20:15:05  raasch
! Initial revision
!
!
! Description:
! ------------
! Initialization of CPU-time measurements for different operating systems
!------------------------------------------------------------------------------!

    USE control_parameters
    USE cpulog

    IMPLICIT NONE

#if defined( __ibm )
    INTEGER(8)         ::  IRTC
#elif defined( __lc )
    INTEGER(8)         ::  count, count_rate
#endif


!
!-- Get initial wall clock time
#if defined( __ibm )

    initial_wallclock_time = IRTC( ) * 1E-9

#elif defined( __lc )

    CALL SYSTEM_CLOCK( count, count_rate )
    initial_wallclock_time = REAL( count ) / REAL( count_rate )

#else
!
!-- So far, nothing is done on these machines
#endif


 END SUBROUTINE local_tremain_ini
