 SUBROUTINE user_check_data_output_pr( variable, var_count, unit )

!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
!
! Former revisions:
! -----------------
! $Id: user_check_data_output_pr.f90 1310 2014-03-14 08:01:56Z raasch $
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 211 2008-11-11 04:46:24Z raasch
! Former file user_interface.f90 split into one file per subroutine
!
! Description:
! ------------
! Set the unit of user defined profile output quantities. For those variables
! not recognized by the user, the parameter unit is set to "illegal", which
! tells the calling routine that the output variable is not defined and leads
! to a program abort.
!------------------------------------------------------------------------------!

    USE arrays_3d
    USE indices
    USE netcdf_control
    USE profil_parameter
    USE statistics
    USE user

    IMPLICIT NONE

    CHARACTER (LEN=*) ::  unit, variable

    INTEGER ::  index, var_count


    SELECT CASE ( TRIM( variable ) )

!
!--    Uncomment and extend the following lines, if necessary.
!--    Add additional CASE statements depending on the number of quantities
!--    for which profiles are to be calculated. The respective calculations
!--    to be performed have to be added in routine user_statistics.
!--    The quantities are (internally) identified by a user-profile-number
!--    (see variable "index" below). The first user-profile must be assigned
!--    the number "pr_palm+1", the second one "pr_palm+2", etc. The respective
!--    user-profile-numbers have also to be used in routine user_statistics!
!       CASE ( 'u*v*' )                      ! quantity string as given in
!                                            ! data_output_pr_user
!          index = pr_palm + 1
!          dopr_index(var_count)  = index    ! quantities' user-profile-number
!          dopr_unit(var_count)   = 'm2/s2'  ! quantity unit
!          hom(:,2,index,:)       = SPREAD( zu, 2, statistic_regions+1 )
!                                            ! grid on which the quantity is
!                                            ! defined (use zu or zw)

       CASE DEFAULT
          unit = 'illegal'

    END SELECT


 END SUBROUTINE user_check_data_output_pr

